export declare class CacheOption<V> {
    static readonly DEFAULT_MAX_SIZE = 100;
    /**
     * Default ttl (seconds) of key-value pair if null ttl is supply when put()
     * If null, the item default never timeout
     */
    private _defaultTTL;
    /**
     * Max num of key-value pair to store.
     */
    private _maxSize;
    /**
     * The map to store key-value pair
     */
    private _store;
    constructor();
    setDefaultTTL(value?: Duration): void;
    setMaxSize(value: Integer): void;
    setStore(value: IMapStorage<V>): void;
    create(): IKeyValueCache<V>;
}

export declare class Duration {
    static readonly microsecondsPerMillisecond = 1000;
    static readonly millisecondsPerSecond = 1000;
    static readonly secondsPerMinute = 60;
    static readonly minutesPerHour = 60;
    static readonly hoursPerDay = 24;
    static readonly microsecondsPerSecond: number;
    static readonly microsecondsPerMinute: number;
    static readonly microsecondsPerHour: number;
    static readonly microsecondsPerDay: number;
    static readonly millisecondsPerMinute: number;
    static readonly millisecondsPerHour: number;
    static readonly millisecondsPerDay: number;
    static readonly secondsPerHour: number;
    static readonly secondsPerDay: number;
    static readonly minutesPerDay: number;
    private readonly _duration;
    constructor({ days, hours, minutes, seconds, milliseconds, microseconds, }: {
        days?: number;
        hours?: number;
        minutes?: number;
        seconds?: number;
        milliseconds?: number;
        microseconds?: number;
    });
    add(other: Duration): Duration;
    subtract(other: Duration): Duration;
    lessThan(other: Duration): boolean;
    greaterThan(other: Duration): boolean;
    lessThanOrEqual(other: Duration): boolean;
    greaterThanOrEqual(other: Duration): boolean;
    get inDays(): number;
    get inHours(): number;
    get inMinutes(): number;
    get inSeconds(): number;
    get inMilliseconds(): number;
    get inMicroseconds(): number;
    equals(other: Duration): boolean;
    compareTo(other: Duration): number;
}

/**
 * A key-value cache with string as key and value of any type.
 * Support
 * - ttl management
 * - size control which remove exceed key-value pair in a FIFO manner
 */
export declare abstract class IKeyValueCache<V> {
    /**
     * Default ttl (seconds) of key-value pair if no ttl is supply when put()
     * If null, never timeout
     */
    protected defaultTTL: Duration | null;
    /**
     * Max num of key-value pair to store.
     */
    protected maxSize: Integer;
    constructor(maxSize: Integer, defaultTTL?: Duration);
    /**
     * Check if this cache is full (>= size).
     * @returns true if is full
     */
    isFull(): boolean;
    /**
     * Return value if found in cache & it is not yet expired.
     * Otherwise, return null
     * @param key
     */
    abstract get(key: string): V | null;
    /**
     * Put the item into cache.
     * If there is already a cache with the same key, this will replace the old one
     * & the expiredTS will be renew by the new ttl.
     *
     * If the cache is full, the first expired item will be deleted.
     * (If no item is expired, the first item in the cache will be deleted)
     * (As long as there is item with expiredTS, the un-expired item will not be deleted)
     *
     * @param ttl optional. If null: use defaultTTL
     * @param noTtl If true: the item will never expire
     */
    abstract put(key: string, value: V, param?: {
        ttl?: Duration;
        noTtl?: boolean;
    }): void;
    /**
     * Delete the item from cache with the given key if exists.
     * @param key key for item to delete
     * @returns true if the item exists and is deleted. Otherwise, false
     */
    abstract delete(key: string): boolean;
    /**
     * Delete all items in this cache.
     */
    abstract clear(): void;
    /**
     * Return the total number of item in this cache
     */
    abstract size(): Integer;
    /**
     * Delete all expired items in the cache.
     * Can be call to reduce cache size.
     */
    abstract clearExpiredItems(): void;
    /**
     * Delete the first expired item in the cache.
     */
    abstract deleteFirstExpiredItem(): void;
}

/**
 * Interface IMapStorage to abstract Map-like object methods
 * so that it support external storage
 */
export declare interface IMapStorage<V> {
    get(key: string): V | null;
    set(key: string, value: V): void;
    has(key: string): boolean;
    delete(key: string): boolean;
    clear(): void;
    size(): Integer;
}

export declare type Integer = number;

export { }
